# Generated by roxygen2: do not edit by hand

S3method(bayes_R2,default)
S3method(loo_pit,default)
S3method(posterior_interval,default)
S3method(predictive_error,default)
S3method(predictive_interval,default)
S3method(prior_summary,default)
export(bayes_R2)
export(init_cpp)
export(log_lik)
export(loo_R2)
export(loo_epred)
export(loo_linpred)
export(loo_pit)
export(loo_predict)
export(loo_predictive_interval)
export(nsamples)
export(posterior_epred)
export(posterior_interval)
export(posterior_linpred)
export(posterior_predict)
export(predictive_error)
export(predictive_interval)
export(prior_summary)
export(rstan_config)
export(rstan_create_package)
export(rstantools_load_code)
export(use_rstan)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(stats,quantile)
importFrom(stats,var)
