#' The 'RSTAN_PACKAGE_NAME' package.
#'
#' @description A DESCRIPTION OF THE PACKAGE
#'
#' @docType package
#' @name RSTAN_PACKAGE_NAME-package
#' @aliases RSTAN_PACKAGE_NAME
#' @useDynLib RSTAN_PACKAGE_NAME, .registration = TRUE
#' @import methods
#' @import Rcpp
#' @importFrom rstan sampling
#' @importFrom rstantools rstan_config
#' @importFrom RcppParallel RcppParallelLibs
#'
#' @references
#' RSTAN_REFERENCE
#'
NULL
