#' Tools for Developing R Packages Interfacing with Stan
#'
#' @name RStanTest
#' @docType package
#' @import Rcpp
#' @import methods
#' @importFrom rstan sampling
#' @useDynLib RStanTest, .registration = TRUE
NULL
